@@ -1,16 +1,14 @@
 /*
- * "$Id: cgi.h 8243 2009-01-14 00:44:55Z mike $"
+ * CGI support library definitions for CUPS.
  *
- *   CGI support library definitions.
+ * Copyright 2007-2010 by Apple Inc.
+ * Copyright 1997-2006 by Easy Software Products.
  *
- *   Copyright 2007-2009 by Apple Inc.
- *   Copyright 1997-2006 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 #ifndef _CUPS_CGI_H_
@@ -33,6 +31,14 @@
 #  include "help-index.h"
 
 
+/*
+ * C++ magic...
+ */
+
+#  ifdef __cplusplus
+extern "C" {
+#  endif /* __cplusplus */
+
 /*
  * Types...
  */
@@ -54,17 +60,19 @@ typedef struct cgi_file_s		/**** Uploaded file data ****/
 extern void		cgiAbort(const char *title, const char *stylesheet,
 			         const char *format, ...);
 extern int		cgiCheckVariables(const char *names);
+extern void		cgiClearVariables(void);
 extern void		*cgiCompileSearch(const char *query);
 extern void		cgiCopyTemplateFile(FILE *out, const char *tmpl);
 extern void		cgiCopyTemplateLang(const char *tmpl);
 extern int		cgiDoSearch(void *search, const char *text);
 extern void		cgiEndHTML(void);
 extern void		cgiEndMultipart(void);
-extern char		*cgiFormEncode(char *dst, const char *src, size_t dstsize);
+extern char		*cgiFormEncode(char *dst, const char *src,
+			               size_t dstsize);
 extern void		cgiFreeSearch(void *search);
 extern const char	*cgiGetArray(const char *name, int element);
 extern void		cgiGetAttributes(ipp_t *request, const char *tmpl);
-extern char		*cgiGetCookie(const char *name, char *buf, int buflen);
+extern const char	*cgiGetCookie(const char *name);
 extern const cgi_file_t	*cgiGetFile(void);
 extern cups_array_t	*cgiGetIPPObjects(ipp_t *response, void *search);
 extern int		cgiGetSize(const char *name);
@@ -98,8 +106,8 @@ extern void		cgiStartMultipart(void);
 extern int		cgiSupportsMultipart(void);
 extern const char	*cgiText(const char *message);
 
-#endif /* !_CUPS_CGI_H_ */
+#  ifdef __cplusplus
+}
+#  endif /* __cplusplus */
 
-/*
- * End of "$Id: cgi.h 8243 2009-01-14 00:44:55Z mike $".
- */
+#endif /* !_CUPS_CGI_H_ */
