@@ -1,16 +1,14 @@
 /*
- * "$Id: config.h 9120 2010-04-23 18:56:34Z mike $"
+ * Configuration file for CUPS on Windows.
  *
- *   Configuration file for CUPS.
+ * Copyright 2007-2016 by Apple Inc.
+ * Copyright 1997-2007 by Easy Software Products.
  *
- *   Copyright 2007-2010 by Apple Inc.
- *   Copyright 1997-2007 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 #ifndef _CUPS_CONFIG_H_
@@ -24,8 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <ctype.h>
 #include <io.h>
+#include <direct.h>
 
 
 /*
@@ -42,8 +40,10 @@
 #define close		_close
 #define fileno		_fileno
 #define lseek		_lseek
+#define mkdir(d,p)	_mkdir(d)
 #define open		_open
 #define read	        _read
+#define rmdir		_rmdir
 #define snprintf 	_snprintf
 #define strdup		_strdup
 #define unlink		_unlink
@@ -51,10 +51,20 @@
 #define write		_write
 
 
+/*
+ * Map the POSIX strcasecmp() and strncasecmp() functions to the Win32 stricmp()
+ * and strnicmp() functions...
+ */
+
+#define strcasecmp	stricmp
+#define strncasecmp	strnicmp
+
+
 /*
  * Map the POSIX sleep() and usleep() functions to the Win32 Sleep() function...
  */
 
+typedef unsigned long useconds_t;
 #define sleep(X)	Sleep(1000 * (X))
 #define usleep(X)	Sleep((X)/1000)
 
@@ -68,7 +78,7 @@
 #  define R_OK		04
 #  define O_RDONLY	_O_RDONLY
 #  define O_WRONLY	_O_WRONLY
-#  define O_CREATE	_O_CREAT
+#  define O_CREAT	_O_CREAT
 #  define O_TRUNC	_O_TRUNC
 
 
@@ -84,8 +94,8 @@
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v1.4.4"
-#define CUPS_MINIMAL "CUPS/1.4.4"
+#define CUPS_SVERSION "CUPS v2.2"
+#define CUPS_MINIMAL "CUPS/2.2"
 
 
 /*
@@ -102,7 +112,7 @@
  * Default file permissions...
  */
 
-#define CUPS_DEFAULT_CONFIG_FILE_PERM 0644
+#define CUPS_DEFAULT_CONFIG_FILE_PERM 0640
 #define CUPS_DEFAULT_LOG_FILE_PERM 0644
 
 
@@ -126,12 +136,8 @@
  */
 
 #define CUPS_DEFAULT_BROWSING 1
-#define CUPS_DEFAULT_BROWSE_LOCAL_PROTOCOLS "CUPS dnssd"
-#define CUPS_DEFAULT_BROWSE_REMOTE_PROTOCOLS ""
-#define CUPS_DEFAULT_BROWSE_SHORT_NAMES 1
+#define CUPS_DEFAULT_BROWSE_LOCAL_PROTOCOLS ""
 #define CUPS_DEFAULT_DEFAULT_SHARED 1
-#define CUPS_DEFAULT_IMPLICIT_CLASSES 1
-#define CUPS_DEFAULT_USE_NETWORK_DEFAULT 0
 
 
 /*
@@ -164,12 +170,19 @@
 
 
 /*
- * Do we have domain socket support?
+ * Do we have domain socket support, and if so what is the default one?
  */
 
 #undef CUPS_DEFAULT_DOMAINSOCKET
 
 
+/*
+ * Default WebInterface value...
+ */
+
+#undef CUPS_DEFAULT_WEBIF
+
+
 /*
  * Where are files stored?
  *
@@ -192,23 +205,25 @@
 
 
 /*
- * Do we have various image libraries?
+ * Do we have posix_spawn?
  */
 
-/* #undef HAVE_LIBPNG */
-/* #undef HAVE_LIBZ */
-/* #undef HAVE_LIBJPEG */
-/* #undef HAVE_LIBTIFF */
+/* #undef HAVE_POSIX_SPAWN */
+
+
+/*
+ * Do we have ZLIB?
+ */
+
+#define HAVE_LIBZ 1
+#define HAVE_INFLATECOPY 1
 
 
 /*
  * Do we have PAM stuff?
  */
 
-#ifndef HAVE_LIBPAM
 #define HAVE_LIBPAM 0
-#endif /* !HAVE_LIBPAM */
-
 /* #undef HAVE_PAM_PAM_APPL_H */
 /* #undef HAVE_PAM_SET_ITEM */
 /* #undef HAVE_PAM_SETCRED */
@@ -229,10 +244,10 @@
 
 
 /*
- * Do we have <scsi/sg.h>?
+ * Use <stdint.h>?
  */
 
-/* #undef HAVE_SCSI_SG_H */
+/* #undef HAVE_STDINT_H */
 
 
 /*
@@ -275,8 +290,6 @@
  */
 
 #define HAVE_STRDUP 1
-#define HAVE_STRCASECMP 1
-#define HAVE_STRNCASECMP 1
 /* #undef HAVE_STRLCAT */
 /* #undef HAVE_STRLCPY */
 
@@ -288,6 +301,13 @@
 /* #undef HAVE_GETEUID */
 
 
+/*
+ * Do we have the setpgid() function?
+ */
+
+/* #undef HAVE_SETPGID */
+
+
 /*
  * Do we have the vsyslog() function?
  */
@@ -347,8 +367,22 @@
 
 /* #undef HAVE_CDSASSL */
 /* #undef HAVE_GNUTLS */
-/* #undef HAVE_LIBSSL */
-/* #undef HAVE_SSL */
+#define HAVE_SSPISSL
+#define HAVE_SSL
+
+
+/*
+ * Do we have the gnutls_transport_set_pull_timeout_function function?
+ */
+
+/* #undef HAVE_GNUTLS_TRANSPORT_SET_PULL_TIMEOUT_FUNCTION */
+
+
+/*
+ * Do we have the gnutls_priority_set_direct function?
+ */
+
+/* #undef HAVE_GNUTLS_PRIORITY_SET_DIRECT */
 
 
 /*
@@ -356,36 +390,35 @@
  */
 
 /* #undef HAVE_AUTHORIZATION_H */
+/* #undef HAVE_SECBASEPRIV_H */
+/* #undef HAVE_SECCERTIFICATE_H */
+/* #undef HAVE_SECIDENTITYSEARCHPRIV_H */
+/* #undef HAVE_SECITEM_H */
+/* #undef HAVE_SECITEMPRIV_H */
 /* #undef HAVE_SECPOLICY_H */
 /* #undef HAVE_SECPOLICYPRIV_H */
-/* #undef HAVE_SECBASEPRIV_H */
-/* #undef HAVE_SECIDENTITYSEARCHPRIV_H */
+/* #undef HAVE_SECURETRANSPORTPRIV_H */
 
 
 /*
- * Do we have the SecIdentitySearchCreateWithPolicy function?
+ * Do we have the cssmErrorString function?
  */
 
-/* #undef HAVE_SECIDENTITYSEARCHCREATEWITHPOLICY */
+/* #undef HAVE_CSSMERRORSTRING */
 
 
 /*
- * Do we have the SLP library?
+ * Do we have the SecGenerateSelfSignedCertificate function?
  */
 
-/* #undef HAVE_LIBSLP */
+/* #undef HAVE_SECGENERATESELFSIGNEDCERTIFICATE */
 
 
 /*
- * Do we have an LDAP library?
+ * Do we have the SecKeychainOpen function?
  */
 
-/* #undef HAVE_LDAP */
-/* #undef HAVE_OPENLDAP */
-/* #undef HAVE_MOZILLA_LDAP */
-/* #undef HAVE_LDAP_SSL_H */
-/* #undef HAVE_LDAP_SSL */
-/* #undef HAVE_LDAP_REBIND_PROC */
+/* #undef HAVE_SECKEYCHAINOPEN */
 
 
 /*
@@ -396,17 +429,31 @@
 
 
 /*
- * Do we have DNS Service Discovery (aka Bonjour)?
+ * Do we have mDNSResponder for DNS Service Discovery (aka Bonjour)?
  */
 
-/* #undef HAVE_DNSSD */
+#define HAVE_DNSSD 1
+
+
+/*
+ * Do we have Avahi for DNS Service Discovery (aka Bonjour)?
+ */
+
+#undef HAVE_AVAHI
 
 
 /*
  * Do we have <sys/ioctl.h>?
  */
 
-/* #undef HAVE_SYS_IOCTL_H */
+#undef HAVE_SYS_IOCTL_H
+
+
+/*
+ * Does the "stat" structure contain the "st_gen" member?
+ */
+
+/* #undef HAVE_ST_GEN */
 
 
 /*
@@ -479,13 +526,6 @@
 /* #undef HAVE_STRUCT_SOCKADDR_SA_LEN */
 
 
-/*
- * Do we have the AIX usersec.h header file?
- */
-
-/* #undef HAVE_USERSEC_H */
-
-
 /*
  * Do we have pthread support?
  */
@@ -493,6 +533,13 @@
 /* #undef HAVE_PTHREAD_H */
 
 
+/*
+ * Do we have on-demand support (launchd/systemd/upstart)?
+ */
+
+/* #undef HAVE_ONDEMAND */
+
+
 /*
  * Do we have launchd support?
  */
@@ -501,6 +548,20 @@
 /* #undef HAVE_LAUNCHD */
 
 
+/*
+ * Do we have systemd support?
+ */
+
+/* #undef HAVE_SYSTEMD */
+
+
+/*
+ * Do we have upstart support?
+ */
+
+/* #undef HAVE_UPSTART */
+
+
 /*
  * Various scripting languages...
  */
@@ -515,30 +576,6 @@
 #define CUPS_PYTHON	""
 
 
-/*
- * Location of the poppler/Xpdf pdftops program...
- */
-
-/* #undef HAVE_PDFTOPS */
-#define CUPS_PDFTOPS ""
-
-
-/*
- * Location of the Ghostscript gs program...
- */
-
-/* #undef HAVE_GHOSTSCRIPT */
-#define CUPS_GHOSTSCRIPT ""
-
-
-/*
- * Do we have Darwin's CoreFoundation and SystemConfiguration frameworks?
- */
-
-/* #undef HAVE_COREFOUNDATION */
-/* #undef HAVE_SYSTEMCONFIGURATION */
-
-
 /*
  * Do we have CoreFoundation public and private headers?
  */
@@ -548,6 +585,13 @@
 /* #undef HAVE_CFBUNDLEPRIV_H */
 
 
+/*
+ * Do we have ApplicationServices public headers?
+ */
+
+/* #undef HAVE_APPLICATIONSERVICES_H */
+
+
 /*
  * Do we have the SCDynamicStoreCopyComputerName function?
  */
@@ -556,7 +600,14 @@
 
 
 /*
- * Do we have MacOSX 10.4's mbr_XXX functions()?
+ * Do we have the getgrouplist() function?
+ */
+
+#undef HAVE_GETGROUPLIST
+
+
+/*
+ * Do we have macOS 10.4's mbr_XXX functions?
  */
 
 /* #undef HAVE_MEMBERSHIP_H */
@@ -565,7 +616,7 @@
 
 
 /*
- * Do we have Darwin's notify_post() header and function?
+ * Do we have Darwin's notify_post header and function?
  */
 
 /* #undef HAVE_NOTIFY_H */
@@ -578,30 +629,20 @@
 
 /* #undef HAVE_DBUS */
 /* #undef HAVE_DBUS_MESSAGE_ITER_INIT_APPEND */
-
-
-/*
- * Do we have the AppleTalk/at_proto.h header?
- */
-
-/* #undef HAVE_APPLETALK_AT_PROTO_H */
+/* #undef HAVE_DBUS_THREADS_INIT */
 
 
 /*
  * Do we have the GSSAPI support library (for Kerberos support)?
  */
 
-/* #undef HAVE_GSSAPI */
-/* #undef HAVE_GSSAPI_H */
-/* #undef HAVE_GSSAPI_GSSAPI_H */
-/* #undef HAVE_GSSAPI_GSSAPI_GENERIC_H */
-/* #undef HAVE_GSSAPI_GSSAPI_KRB5_H */
-/* #undef HAVE_GSSKRB5_REGISTER_ACCEPTOR_IDENTITY */
+/* #undef HAVE_GSS_ACQUIRE_CRED_EX_F */
 /* #undef HAVE_GSS_C_NT_HOSTBASED_SERVICE */
-/* #undef HAVE_KRB5_CC_NEW_UNIQUE */
-/* #undef HAVE_KRB5_IPC_CLIENT_SET_TARGET_UID */
-/* #undef HAVE_KRB5_H */
-/* #undef HAVE_HEIMDAL */
+/* #undef HAVE_GSS_GSSAPI_H */
+/* #undef HAVE_GSS_GSSAPI_SPI_H */
+/* #undef HAVE_GSSAPI */
+/* #undef HAVE_GSSAPI_GSSAPI_H */
+/* #undef HAVE_GSSAPI_H */
 
 
 /*
@@ -689,7 +730,7 @@
  * Do we have libusb?
  */
 
-/* #undef HAVE_USB_H */
+/* #undef HAVE_LIBUSB */
 
 
 /*
@@ -699,8 +740,62 @@
 /* #undef HAVE_TCPD_H */
 
 
-#endif /* !_CUPS_CONFIG_H_ */
+/*
+ * Do we have <iconv.h>?
+ */
+
+/* #undef HAVE_ICONV_H */
+
 
 /*
- * End of "$Id: config.h 9120 2010-04-23 18:56:34Z mike $".
+ * Do we have statfs or statvfs and one of the corresponding headers?
  */
+
+/* #undef HAVE_STATFS */
+/* #undef HAVE_STATVFS */
+/* #undef HAVE_SYS_MOUNT_H */
+/* #undef HAVE_SYS_STATFS_H */
+/* #undef HAVE_SYS_STATVFS_H */
+/* #undef HAVE_SYS_VFS_H */
+
+
+/*
+ * Location of macOS localization bundle, if any.
+ */
+
+/* #undef CUPS_BUNDLEDIR */
+
+
+/*
+ * Do we have XPC?
+ */
+
+/* #undef HAVE_XPC */
+/* #undef HAVE_XPC_PRIVATE_H */
+
+
+/*
+ * Do we have Mini-XML?
+ */
+
+/* #undef HAVE_MXML_H */
+
+
+/*
+ * Do we have the C99 abs() function?
+ */
+
+/* #undef HAVE_ABS */
+#if !defined(HAVE_ABS) && !defined(abs)
+#  if defined(__GNUC__) || __STDC_VERSION__ >= 199901L
+#    define abs(x) _cups_abs(x)
+static inline int _cups_abs(int i) { return (i < 0 ? -i : i); }
+#  elif defined(_MSC_VER)
+#    define abs(x) _cups_abs(x)
+static __inline int _cups_abs(int i) { return (i < 0 ? -i : i); }
+#  else
+#    define abs(x) ((x) < 0 ? -(x) : (x))
+#  endif /* __GNUC__ || __STDC_VERSION__ */
+#endif /* !HAVE_ABS && !abs */
+
+#endif /* !_CUPS_CONFIG_H_ */
